r"""
Script core module contain the logic of the script functions and basically 
mirror the content of the scripts before the appearence of script-core, 
therefore the scripts from scripts folder are just the bindings to the 
corresponding files in script-core.
"""
